[−][src]Struct prost_build::Service
A service descriptor.
Fields
name: StringThe service name in Rust style.
proto_name: StringThe service name as it appears in the .proto file.
package: StringThe package name as it appears in the .proto file.
comments: CommentsThe service comments.
methods: Vec<Method>The service methods.
options: ServiceOptionsThe service options.