Struct prost_build::Module
source · pub struct Module { /* private fields */ }Expand description
A Rust module path for a Protobuf package.
Implementations
sourceimpl Module
impl Module
sourcepub fn from_parts<I>(parts: I) -> Selfwhere
I: IntoIterator,
I::Item: Into<String>,
pub fn from_parts<I>(parts: I) -> Selfwhere
I: IntoIterator,
I::Item: Into<String>,
Construct a module path from an iterator of parts.
sourcepub fn from_protobuf_package_name(name: &str) -> Self
pub fn from_protobuf_package_name(name: &str) -> Self
Construct a module path from a Protobuf package name.
Constituent parts are automatically converted to snake case in order to follow Rust module naming conventions.
Examples found in repository?
src/lib.rs (line 919)
810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968
pub fn compile_protos(
&mut self,
protos: &[impl AsRef<Path>],
includes: &[impl AsRef<Path>],
) -> Result<()> {
let mut target_is_env = false;
let target: PathBuf = self.out_dir.clone().map(Ok).unwrap_or_else(|| {
env::var_os("OUT_DIR")
.ok_or_else(|| {
Error::new(ErrorKind::Other, "OUT_DIR environment variable is not set")
})
.map(|val| {
target_is_env = true;
Into::into(val)
})
})?;
// TODO: This should probably emit 'rerun-if-changed=PATH' directives for cargo, however
// according to [1] if any are output then those paths replace the default crate root,
// which is undesirable. Figure out how to do it in an additive way; perhaps gcc-rs has
// this figured out.
// [1]: http://doc.crates.io/build-script.html#outputs-of-the-build-script
let tmp;
let file_descriptor_set_path = if let Some(path) = &self.file_descriptor_set_path {
path.clone()
} else {
if self.skip_protoc_run {
return Err(Error::new(
ErrorKind::Other,
"file_descriptor_set_path is required with skip_protoc_run",
));
}
tmp = tempfile::Builder::new().prefix("prost-build").tempdir()?;
tmp.path().join("prost-descriptor-set")
};
if !self.skip_protoc_run {
let protoc = protoc_from_env();
let mut cmd = Command::new(protoc.clone());
cmd.arg("--include_imports")
.arg("--include_source_info")
.arg("-o")
.arg(&file_descriptor_set_path);
for include in includes {
if include.as_ref().exists() {
cmd.arg("-I").arg(include.as_ref());
} else {
debug!(
"ignoring {} since it does not exist.",
include.as_ref().display()
)
}
}
// Set the protoc include after the user includes in case the user wants to
// override one of the built-in .protos.
if let Some(protoc_include) = protoc_include_from_env() {
cmd.arg("-I").arg(protoc_include);
}
for arg in &self.protoc_args {
cmd.arg(arg);
}
for proto in protos {
cmd.arg(proto.as_ref());
}
debug!("Running: {:?}", cmd);
let output = cmd.output().map_err(|error| {
Error::new(
error.kind(),
format!("failed to invoke protoc (hint: https://docs.rs/prost-build/#sourcing-protoc): (path: {:?}): {}", &protoc, error),
)
})?;
if !output.status.success() {
return Err(Error::new(
ErrorKind::Other,
format!("protoc failed: {}", String::from_utf8_lossy(&output.stderr)),
));
}
}
let buf = fs::read(&file_descriptor_set_path).map_err(|e| {
Error::new(
e.kind(),
format!(
"unable to open file_descriptor_set_path: {:?}, OS: {}",
&file_descriptor_set_path, e
),
)
})?;
let file_descriptor_set = FileDescriptorSet::decode(&*buf).map_err(|error| {
Error::new(
ErrorKind::InvalidInput,
format!("invalid FileDescriptorSet: {}", error),
)
})?;
let requests = file_descriptor_set
.file
.into_iter()
.map(|descriptor| {
(
Module::from_protobuf_package_name(descriptor.package()),
descriptor,
)
})
.collect::<Vec<_>>();
let file_names = requests
.iter()
.map(|req| {
(
req.0.clone(),
req.0.to_file_name_or(&self.default_package_filename),
)
})
.collect::<HashMap<Module, String>>();
let modules = self.generate(requests)?;
for (module, content) in &modules {
let file_name = file_names
.get(module)
.expect("every module should have a filename");
let output_path = target.join(file_name);
let previous_content = fs::read(&output_path);
if previous_content
.map(|previous_content| previous_content == content.as_bytes())
.unwrap_or(false)
{
trace!("unchanged: {:?}", file_name);
} else {
trace!("writing: {:?}", file_name);
fs::write(output_path, content)?;
}
}
if let Some(ref include_file) = self.include_file {
trace!("Writing include file: {:?}", target.join(include_file));
let mut file = fs::File::create(target.join(include_file))?;
self.write_includes(
modules.keys().collect(),
&mut file,
0,
if target_is_env { None } else { Some(&target) },
)?;
file.flush()?;
}
Ok(())
}sourcepub fn to_file_name_or(&self, default: &str) -> String
pub fn to_file_name_or(&self, default: &str) -> String
Format the module path into a filename for generated Rust code.
If the module path is empty, default is used to provide the root of the filename.
Examples found in repository?
src/lib.rs (line 930)
810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968
pub fn compile_protos(
&mut self,
protos: &[impl AsRef<Path>],
includes: &[impl AsRef<Path>],
) -> Result<()> {
let mut target_is_env = false;
let target: PathBuf = self.out_dir.clone().map(Ok).unwrap_or_else(|| {
env::var_os("OUT_DIR")
.ok_or_else(|| {
Error::new(ErrorKind::Other, "OUT_DIR environment variable is not set")
})
.map(|val| {
target_is_env = true;
Into::into(val)
})
})?;
// TODO: This should probably emit 'rerun-if-changed=PATH' directives for cargo, however
// according to [1] if any are output then those paths replace the default crate root,
// which is undesirable. Figure out how to do it in an additive way; perhaps gcc-rs has
// this figured out.
// [1]: http://doc.crates.io/build-script.html#outputs-of-the-build-script
let tmp;
let file_descriptor_set_path = if let Some(path) = &self.file_descriptor_set_path {
path.clone()
} else {
if self.skip_protoc_run {
return Err(Error::new(
ErrorKind::Other,
"file_descriptor_set_path is required with skip_protoc_run",
));
}
tmp = tempfile::Builder::new().prefix("prost-build").tempdir()?;
tmp.path().join("prost-descriptor-set")
};
if !self.skip_protoc_run {
let protoc = protoc_from_env();
let mut cmd = Command::new(protoc.clone());
cmd.arg("--include_imports")
.arg("--include_source_info")
.arg("-o")
.arg(&file_descriptor_set_path);
for include in includes {
if include.as_ref().exists() {
cmd.arg("-I").arg(include.as_ref());
} else {
debug!(
"ignoring {} since it does not exist.",
include.as_ref().display()
)
}
}
// Set the protoc include after the user includes in case the user wants to
// override one of the built-in .protos.
if let Some(protoc_include) = protoc_include_from_env() {
cmd.arg("-I").arg(protoc_include);
}
for arg in &self.protoc_args {
cmd.arg(arg);
}
for proto in protos {
cmd.arg(proto.as_ref());
}
debug!("Running: {:?}", cmd);
let output = cmd.output().map_err(|error| {
Error::new(
error.kind(),
format!("failed to invoke protoc (hint: https://docs.rs/prost-build/#sourcing-protoc): (path: {:?}): {}", &protoc, error),
)
})?;
if !output.status.success() {
return Err(Error::new(
ErrorKind::Other,
format!("protoc failed: {}", String::from_utf8_lossy(&output.stderr)),
));
}
}
let buf = fs::read(&file_descriptor_set_path).map_err(|e| {
Error::new(
e.kind(),
format!(
"unable to open file_descriptor_set_path: {:?}, OS: {}",
&file_descriptor_set_path, e
),
)
})?;
let file_descriptor_set = FileDescriptorSet::decode(&*buf).map_err(|error| {
Error::new(
ErrorKind::InvalidInput,
format!("invalid FileDescriptorSet: {}", error),
)
})?;
let requests = file_descriptor_set
.file
.into_iter()
.map(|descriptor| {
(
Module::from_protobuf_package_name(descriptor.package()),
descriptor,
)
})
.collect::<Vec<_>>();
let file_names = requests
.iter()
.map(|req| {
(
req.0.clone(),
req.0.to_file_name_or(&self.default_package_filename),
)
})
.collect::<HashMap<Module, String>>();
let modules = self.generate(requests)?;
for (module, content) in &modules {
let file_name = file_names
.get(module)
.expect("every module should have a filename");
let output_path = target.join(file_name);
let previous_content = fs::read(&output_path);
if previous_content
.map(|previous_content| previous_content == content.as_bytes())
.unwrap_or(false)
{
trace!("unchanged: {:?}", file_name);
} else {
trace!("writing: {:?}", file_name);
fs::write(output_path, content)?;
}
}
if let Some(ref include_file) = self.include_file {
trace!("Writing include file: {:?}", target.join(include_file));
let mut file = fs::File::create(target.join(include_file))?;
self.write_includes(
modules.keys().collect(),
&mut file,
0,
if target_is_env { None } else { Some(&target) },
)?;
file.flush()?;
}
Ok(())
}sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
The number of parts in the module’s path.
Examples found in repository?
src/lib.rs (line 999)
970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028
fn write_includes(
&self,
mut entries: Vec<&Module>,
outfile: &mut fs::File,
depth: usize,
basepath: Option<&PathBuf>,
) -> Result<usize> {
let mut written = 0;
entries.sort();
while !entries.is_empty() {
let modident = entries[0].part(depth);
let matching: Vec<&Module> = entries
.iter()
.filter(|&v| v.part(depth) == modident)
.copied()
.collect();
{
// Will NLL sort this mess out?
let _temp = entries
.drain(..)
.filter(|&v| v.part(depth) != modident)
.collect();
entries = _temp;
}
self.write_line(outfile, depth, &format!("pub mod {} {{", modident))?;
let subwritten = self.write_includes(
matching
.iter()
.filter(|v| v.len() > depth + 1)
.copied()
.collect(),
outfile,
depth + 1,
basepath,
)?;
written += subwritten;
if subwritten != matching.len() {
let modname = matching[0].to_partial_file_name(..=depth);
if basepath.is_some() {
self.write_line(
outfile,
depth + 1,
&format!("include!(\"{}.rs\");", modname),
)?;
} else {
self.write_line(
outfile,
depth + 1,
&format!("include!(concat!(env!(\"OUT_DIR\"), \"/{}.rs\"));", modname),
)?;
}
written += 1;
}
self.write_line(outfile, depth, "}")?;
}
Ok(written)
}Trait Implementations
sourceimpl Ord for Module
impl Ord for Module
sourceimpl PartialEq<Module> for Module
impl PartialEq<Module> for Module
sourceimpl PartialOrd<Module> for Module
impl PartialOrd<Module> for Module
1.0.0 · sourcefn le(&self, other: &Rhs) -> bool
fn le(&self, other: &Rhs) -> bool
This method tests less than or equal to (for
self and other) and is used by the <=
operator. Read moreimpl Eq for Module
impl StructuralEq for Module
impl StructuralPartialEq for Module