Struct prost_build::Module
source · pub struct Module { /* private fields */ }Expand description
A Rust module path for a Protobuf package.
Implementations
sourceimpl Module
impl Module
sourcepub fn from_parts<I>(parts: I) -> Selfwhere
I: IntoIterator,
I::Item: Into<String>,
pub fn from_parts<I>(parts: I) -> Selfwhere
I: IntoIterator,
I::Item: Into<String>,
Construct a module path from an iterator of parts.
sourcepub fn from_protobuf_package_name(name: &str) -> Self
pub fn from_protobuf_package_name(name: &str) -> Self
Construct a module path from a Protobuf package name.
Constituent parts are automatically converted to snake case in order to follow Rust module naming conventions.
sourcepub fn to_file_name_or(&self, default: &str) -> String
pub fn to_file_name_or(&self, default: &str) -> String
Format the module path into a filename for generated Rust code.
If the module path is empty, default is used to provide the root of the filename.